import websockets
from . import events_pb2

### LiveApex Lobby Functions ###
# These functions are used to interact with the lobby and players in the custom match #

class Lobby:
    """
    # Lobby

    This class contains functions to alter or get data on the lobby and it's players.
    """

    async def sendChatMessage(text: str):
        """
        # Send a Chat Message

        This function sends a chat message to the lobby.

        ## Parameters

        :text: The text of the chat message.

        ## Notes

        :sendChatMessage: has a rate limit of ~10 messages in quick succession, any messages after this limit will be ignored by the game.

        ## Example

        ```python
        await LiveApex.Lobby.sendChatMessage('Hello World!')
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SendChat.CopyFrom(events_pb2.CustomMatch_SendChat(text=text))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def togglePause(countdown: int):
        """
        # Toggle Pause

        This function toggles the pause state of the custom match.

        ## Parameters

        :countdown: The countdown until the match is paused/unpaused. If set to 0, the match pause state will change instantly.

        ## Example

        ```python
        await LiveApex.Lobby.togglePause(5)
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_TogglePause.CopyFrom(events_pb2.CustomMatch_TogglePause(preTimer=countdown))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def createLobby():
        """
        # Create Lobby

        This function creates a custom match lobby.

        ## Example

        ```python
        await LiveApex.Lobby.createLobby()
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_CreateLobby.CopyFrom(events_pb2.CustomMatch_CreateLobby())

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def joinLobby(lobby_code: str):
        """
        # Join Lobby

        This function joins a custom match lobby.

        ## Parameters

        :lobby_code: The lobby code to join (either admin or player code works).

        ## Example

        ```python
        await LiveApex.Lobby.joinLobby(abcd1234)
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_JoinLobby.CopyFrom(events_pb2.CustomMatch_JoinLobby(roleToken=lobby_code))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def leaveLobby():
        """
        # Leave Lobby

        This function leaves a custom match lobby.

        ## Example

        ```python
        await LiveApex.Lobby.leaveLobby()
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_LeaveLobby.CopyFrom(events_pb2.CustomMatch_LeaveLobby())

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def setReady(ready: bool):
        """
        # Set Ready

        This function sets the ready state of the player.

        ## Parameters

        :ready: The ready state of the player.

        ## Example

        ```python
        await LiveApex.Lobby.setReady(True)
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetReady.CopyFrom(events_pb2.CustomMatch_SetReady(isReady=ready))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def setTeamName(team_id: int, team_name: str):
        """
        # Set Team Name

        This function sets the name of a team.

        ## Parameters

        :team_id: The ID of the team. team_id=0 is unassigned, team_id=1 is observer, team_id=2 is team 1 and so on.
        :team_name: The name of the team.

        ## Notes

        Team names can only be set when using lobby codes from EA/Respawn.

        ## Raises

        `notInLobby` - Not connected to a custom match.\n
        `notInGame` - Game client is not running Apex Legends or is not past the main menu.\n
        `invalidTeamIndex` - Invalid team index, must be a value between 1 and 21 (Differs gamemode to gamemode).

        ## Example

        ```python
        await LiveApex.Lobby.setTeamName(2, 'Awesome Team')
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetTeamName.CopyFrom(events_pb2.CustomMatch_SetTeamName(teamId=team_id, teamName=team_name))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def getPlayers():
        """
        # Get Custom Match Players

        This function gets the custom match players.

        ## Example

        ```python
        await LiveApex.Lobby.getPlayers()
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_GetLobbyPlayers.CopyFrom(events_pb2.CustomMatch_GetLobbyPlayers())

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def movePlayer(team_id: int, hardware_name: str, user_hash: str):
        """
        # Set Team

        This function moves a player to a different team.

        ## Parameters

        :team_id: The ID of the team. team_id=0 is unassigned, team_id=1 is observer, team_id=2 is team 1 and so on.
        :hardware_name: The platform of the player, i.e PC-STEAM.
        :user_hash: The hash of the player. Obtained via LiveApex.Lobby.getPlayers().

        ## Example

        ```python
        await LiveApex.Lobby.movePlayer(2, 'PC-STEAM', 'ad431d95fd8cdaf5e56f2b661cada2fb')
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetTeam.CopyFrom(events_pb2.CustomMatch_SetTeam(teamId=team_id, targetHardwareName=hardware_name, targetNucleusHash=user_hash))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def kickPlayer(hardware_name: str, user_hash: str):
        """
        # Kick Player

        This function kicks a player from the custom match.

        ## Parameters

        :hardware_name: The platform of the player, i.e PC-STEAM.
        :user_hash: The hash of the player.

        ## Example

        ```python
        await LiveApex.Lobby.kickPlayer('PC-STEAM', 'ad431d95fd8cdaf5e56f2b661cada2fb')
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_KickPlayer.CopyFrom(events_pb2.CustomMatch_KickPlayer(targetHardwareName=hardware_name, targetNucleusHash=user_hash))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def getSettings():
        """
        # Get Custom Match Settings

        This function gets the custom match settings.

        ## Example

        ```python
        await LiveApex.Lobby.getSettings()
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_GetSettings.CopyFrom(events_pb2.CustomMatch_GetSettings())
            request.withAck = True

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def setSettings(playlist_name: str, admin_chat: bool, team_rename: bool, self_assign: bool, aim_assist: bool, anon_mode: bool):
        """
        # Set Custom Match Settings

        This function sets the custom match settings.

        ## Parameters

        :playlist_name: The name of the playlist.
        :admin_chat: Whether to enable admin chat.
        :team_rename: Whether to enable team renaming.
        :self_assign: Whether to enable self assign.
        :aim_assist: Whether to enable aim assist.
        :anon_mode: Whether to enable anonymous mode.

        ## Example

        ```python
        await LiveApex.Lobby.setSettings(des_hu_cm, True, True, True, False, False)
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetSettings.CopyFrom(events_pb2.CustomMatch_SetSettings(playlistName=playlist_name, adminChat=admin_chat, teamRename=team_rename, selfAssign=self_assign, aimAssist=aim_assist, anonMode=anon_mode))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def setLegendBan(bans: list[str]):
        """
        # Set Legend Ban

        This function sets legend bans for the lobby. To reset the bans, run this function with an empty list.

        ## Parameters

        :bans: A list of legend names to ban.

        ## Example

        ```python
        await LiveApex.Lobby.setLegendBan(['wraith', 'madmaggie'])
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetLegendBan.CopyFrom(events_pb2.CustomMatch_SetLegendBan(legendRefs=bans))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def getLegendBans():
        """
        # Get Legend Bans

        This function gets the legend bans for the lobby.

        ## Example

        ```python
        await LiveApex.Lobby.getLegendBans()
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_GetLegendBanStatus.CopyFrom(events_pb2.CustomMatch_GetLegendBanStatus())

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def startGame(status: bool):
        """
        # Start Game

        This function starts/stops custom match matchmaking.

        ## Parameters

        :status: Whether to start or stop matchmaking.

        ## Example

        ```python
        await LiveApex.Lobby.startGame(True)
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetMatchmaking.CopyFrom(events_pb2.CustomMatch_SetMatchmaking(enabled=status))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def setDropLocation(team_id: int, drop_location: int):
        """
        # Set Drop Location

        This function sets the drop location of a team.

        ## Parameters

        :team_id: The ID of the team. team_id=0 is unassigned, team_id=1 is observer, team_id=2 is team 1 and so on.
        :drop_location: The POI ID of any POI (this is the same system as the @XX that can also be used to set drop locations).

        ## Example

        ```python
        await LiveApex.Lobby.setDropLocation(2, 20)
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            request.customMatch_SetSpawnPoint.CopyFrom(events_pb2.CustomMatch_SetSpawnPoint(teamId=team_id, spawnPoint=drop_location))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def setEndRingExclusions(map_region: str):
        """
        # Set End Ring Exclusions

        This function sets the end ring exclusions for the lobby. Run this function multiple times if you want to exclude multiple regions.

        ## Parameters

        :map_region: The map region to exclude. TOP_LEFT, TOP_RIGHT, BOTTOM_LEFT, BOTTOM_RIGHT, CENTER or REGIONS_COUNT to reset

        ## Example

        ```python
        await LiveApex.Lobby.setEndRingExclusions('TOP_LEFT')
        ```
        """

        uri = 'ws://127.0.0.1:7777'
        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            if map_region == "TOP_LEFT":
                request.customMatch_SetEndRingExclusion.selectionToExclude = events_pb2.MapRegion.TOP_LEFT
            elif map_region == "TOP_RIGHT":
                request.customMatch_SetEndRingExclusion.selectionToExclude = events_pb2.MapRegion.TOP_RIGHT
            elif map_region == "BOTTOM_LEFT":
                request.customMatch_SetEndRingExclusion.selectionToExclude = events_pb2.MapRegion.BOTTOM_LEFT
            elif map_region == "BOTTOM_RIGHT":
                request.customMatch_SetEndRingExclusion.selectionToExclude = events_pb2.MapRegion.BOTTOM_RIGHT
            elif map_region == "CENTER":
                request.customMatch_SetEndRingExclusion.selectionToExclude = events_pb2.MapRegion.CENTER
            elif map_region == "REGIONS_COUNT":
                request.customMatch_SetEndRingExclusion.selectionToExclude = events_pb2.MapRegion.REGIONS_COUNT
            else:
                raise ValueError("[LiveApexLobby] Invalid map region. Must be one of: TOP_LEFT, TOP_RIGHT, BOTTOM_LEFT, BOTTOM_RIGHT, CENTER or REGIONS_COUNT.")

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)

    async def changeCamera(selection: str, input: str):
        """
        # Change Camera

        This function changes the camera of the observer.

        ## Parameters

        :selection: Player name, nucleusHash or use one of the following poi options: NEXT, PREVIOUS, KILL_LEADER, CLOSEST_ENEMY, CLOSEST_PLAYER, LATEST_ATTACKER.
        :input: poi, player or hash.

        ## Example

        ```python
        await LiveApex.Lobby.changeCamera('KILL_LEADER')
        ```
        """

        uri = 'ws://127.0.0.1:7777'

        async with websockets.connect(uri, ping_interval=20, ping_timeout=20) as websocket:
            # Construct the Request message
            request = events_pb2.Request()
            if input == "poi":
                if selection in ["NEXT", "PREVIOUS", "KILL_LEADER", "CLOSEST_ENEMY", "CLOSEST_PLAYER", "LATEST_ATTACKER"]:
                    if selection == "NEXT":
                        request.changeCamera.poi = events_pb2.PlayerOfInterest.NEXT
                    elif selection == "PREVIOUS":
                        request.changeCamera.poi = events_pb2.PlayerOfInterest.PREVIOUS
                    elif selection == "KILL_LEADER":
                        request.changeCamera.poi = events_pb2.PlayerOfInterest.KILL_LEADER
                    elif selection == "CLOSEST_ENEMY":
                        request.changeCamera.poi = events_pb2.PlayerOfInterest.CLOSEST_ENEMY
                    elif selection == "CLOSEST_PLAYER":
                        request.changeCamera.poi = events_pb2.PlayerOfInterest.CLOSEST_PLAYER
                    elif selection == "LATEST_ATTACKER":
                        request.changeCamera.poi = events_pb2.PlayerOfInterest.LATEST_ATTACKER

            elif input == "player":
                request.changeCamera.name = selection

            elif input == "hash":
                request.changeCamera.nucleusHash = selection

            # Construct the Request message
            request = events_pb2.Request()
            request.changeCamera.CopyFrom(events_pb2.ChangeCamera(target=selection))

            # Serialize the Request message
            serialized_request = request.SerializeToString()

            # Send the message
            await websocket.send(serialized_request)


### LiveApex Translator Functions ###
# These functions convert internal to common names or vice versa #

## Internal to Common Names
map_translations = {
    "mp_rr_canyonlands_hu": "Kings Canyon",
    "mp_rr_tropic_island_mu1": "Storm Point (Season 13)",
    "mp_rr_tropic_island_mu1_storm": "Storm Point (Season 18)",
    "mp_rr_tropic_island_mu2": "Storm Point (Season 21)",
    "mp_rr_tropic_island_mu2_landscape": "Storm Point", # Season 25 optimization
    "mp_rr_desertlands_mu3": "Worlds Edge (Season 10)",
    "mp_rr_desertlands_mu4": "Worlds Edge (Season 16)",
    "mp_rr_desertlands_hu": "Worlds Edge",
    "mp_rr_olympus_mu2": "Olympus",
    "mp_rr_divided_moon": "Broken Moon (Season 15)",
    "mp_rr_divided_moon_mu1": "Broken Moon",
    "mp_rr_district": "E-District",
}

datacenter_translations = {
    "ap-east-1": "Hong Kong",
    "ap-northeast-1": "Tokyo",
    "ap-southeast-1": "Singapore",
    "ap-southeast-2": "Sydney",
    "eu-central-1": "Frankfurt",
    "me-south-1": "Bahrain",
    "sa-east-1": "Sao Paolo",
    "us-east-1": "North Virginia",
    "us-east-2": "Ohio",
    "us-west-2": "Oregon",
}

### MISSING MELEE HEIRLOOM VARIANTS
weapon_translations = {
    # GRENADES
    "mp_weapon_grenade_emp": "Arc Star",
    "mp_weapon_thermite_grenade": "Thermite Grenade",
    'mp_weapon_frag_grenade': "Frag Grenade",
    # AR WEAPONS
    "mp_weapon_energy_ar": "Havoc",
    "mp_weapon_vinson": "Flatline",
    "mp_weapon_nemesis": "Nemesis",
    "mp_weapon_rspn101": "R-301",
    "mp_weapon_hemlok": "Hemlok",
    # PISTOL WEAPONS
    "mp_weapon_semipistol": "P2020",
    "mp_weapon_wingman": "Wingman",
    "mp_weapon_autopistol": "RE-45",
    # LMG WEAPONS
    "mp_weapon_dragon_lmg": "Rampage",
    "mp_weapon_lmg": "Spitfire",
    "mp_weapon_esaw": "Devotion",
    "mp_weapon_lstar": "L-STAR",
    # SMG WEAPONS
    "mp_weapon_car": "Car",
    "mp_weapon_r97": "R-99",
    "mp_weapon_volt_smg": "Volt",
    "mp_weapon_pdw": "Prowler",
    'mp_weapon_alternator_smg': "Alternator",
    # SNIPER WEAPONS
    "mp_weapon_dmr": "Longbow",
    "mp_weapon_defender": "Charge Rifle",
    "mp_weapon_sentinel": "Sentinel",
    # MARKSMAN WEAPONS
    "mp_weapon_3030": "30-30",
    "mp_weapon_g2": "G7 Scout",
    "mp_weapon_doubletake": "Triple Take",
    # SHOTGUN WEAPONS
    "mp_weapon_energy_shotgun": "Peacekeeper",
    "mp_weapon_shotgun_pistol": "Mozambique Shotgun",
    "mp_weapon_shotgun": "EVA-8",
    "mp_weapon_mastiff": "Mastiff",
    # CARE PACKAGE WEAPONS
    "mp_weapon_sniper": "Kraber",
    # OTHER WEAPONS
    "mp_weapon_bow": "Bocek",
    "mp_weapon_melee_survival": "Melee",
    'mp_weapon_mounted_turret_weapon': "Sheila (Placed)",
    'mp_weapon_mounted_turret_placeable': "Sheila (Mobile)"
}

## Common to Internal Names
map_untranslations = {
    "Kings Canyon": "mp_rr_canyonlands_hu",
    "Storm Point (Season 13)": "mp_rr_tropic_island_mu1",
    "Storm Point (Season 18)": "mp_rr_tropic_island_mu1_storm",
    "Storm Point (Season 21)": "mp_rr_tropic_island_mu2",
    "Storm Point": "mp_rr_tropic_island_mu2_landscape", # Season 25 optimization
    "Worlds Edge (Season 10)": "mp_rr_desertlands_mu3",
    "Worlds Edge (Season 16)": "mp_rr_desertlands_mu4",
    "Worlds Edge": "mp_rr_desertlands_hu",
    "Olympus": "mp_rr_olympus_mu2",
    "Broken Moon (Season 15)": "mp_rr_divided_moon",
    "Broken Moon": "mp_rr_divided_moon_mu1",
    "E-District": "mp_rr_district"
}

datacenter_untranslations = {
    "Hong Kong": "ap-east-1",
    "Tokyo": "ap-northeast-1",
    "Singapore": "ap-southeast-1",
    "Sydney": "ap-southeast-2",
    "Frankfurt": "eu-central-1",
    "Bahrain": "me-south-1",
    "Sao Paolo": "sa-east-1",
    "North Virginia": "us-east-1",
    "Ohio": "us-east-2",
    "Oregon": "us-west-2"
}

# MISSING MELEE HEIRLOOM VARIANTS
weapon_untranslations = {
    # GRENADES
    "Arc Star": "mp_weapon_grenade_emp",
    "Thermite Grenade": "mp_weapon_thermite_grenade",
    'Frag Grenade': 'mp_weapon_frag_grenade',
    # AR WEAPONS
    "Havoc": "mp_weapon_energy_ar",
    "Flatline": "mp_weapon_vinson",
    "Nemesis": "mp_weapon_nemesis",
    "R-301": "mp_weapon_rspn101",
    "Hemlok": "mp_weapon_hemlok",
    # PISTOL WEAPONS
    "P2020": "mp_weapon_semipistol",
    "Wingman": "mp_weapon_wingman",
    "RE-45": "mp_weapon_autopistol",
    # LMG WEAPONS
    "Rampage": "mp_weapon_dragon_lmg",
    "Spitfire": "mp_weapon_lmg",
    "Devotion": "mp_weapon_esaw",
    "L-STAR": "mp_weapon_lstar",
    # SMG WEAPONS
    "Car": 'mp_weapon_car',
    'R-99': 'mp_weapon_r97',
    'Volt': 'mp_weapon_volt_smg',
    'Prowler': 'mp_weapon_pdw',
    'Alternator': 'mp_weapon_alternator_smg',
    # SNIPER WEAPONS
    'Longbow': 'mp_weapon_dmr',
    'Charge Rifle': 'mp_weapon_defender',
    'Sentinel': 'mp_weapon_sentinel',
    # MARKSMAN WEAPONS
    '30-30': 'mp_weapon_3030',
    'G7 Scout': 'mp_weapon_g2',
    'Triple Take': 'mp_weapon_doubletake',
    # SHOTGUN WEAPONS
    'Peacekeeper': 'mp_weapon_energy_shotgun',
    'Mozambique Shotgun': 'mp_weapon_shotgun_pistol',
    'EVA-8': 'mp_weapon_shotgun',
    'Mastiff': 'mp_weapon_mastiff',
    # CARE PACKAGE WEAPONS
    'Kraber': 'mp_weapon_sniper',
    # OTHER WEAPONS
    'Bocek': 'mp_weapon_bow',
    'Melee': 'mp_weapon_melee_survival',
    'Sheila (Placed)': 'mp_weapon_mounted_turret_weapon',
    'Sheila (Mobile)': 'mp_weapon_mounted_turret_placeable'
}

class Translator:
    """
    # Translator

    This class contains functions to translate data from internal to common names.
    """

    def translateDatacenter(datacenter: str):
        """
        # Translate a datacenter

        This function translates a datacenter from internal reference to a common name.

        ## Parameters

        :datacenter: The datacenter to translate.

        ## Example

        ```python
        LiveApex.Translator.translateDatacenter('')
        ```

        ## Raises

        Exception: {datacenter} | If the datacenter is unknown.
        """

        if datacenter in datacenter_translations:
            translated = datacenter_translations[datacenter]
        else:
            raise Exception(f"Unknown datacenter: {datacenter}")

        return translated

    def translateWeapon(weapon: str):
        """
        # Translate a Weapon

        This function translates a weapon from internal reference to a common name.

        ## Parameters

        :weapon: The weapon to translate.

        ## Example

        ```python
        LiveApex.Translator.translateWeapon('mp_weapon_melee_survival')
        ```

        ## Raises

        Exception: {weapon} | If the weapon is unknown.
        """

        if weapon in weapon_translations:
            translated = weapon_translations[weapon]
        else:
            raise Exception(f"Unknown weapon: {weapon}")

        return translated

    def translateMap(map: str):
        """
        # Translate a Map

        This function translates a map from internal reference to a common name.

        ## Parameters

        :map: The map to translate.

        ## Example

        ```python
        LiveApex.Translator.translateMap('mp_rr_tropic_island_mu2')
        ```

        ## Raises

        Exception: {map} | If the map is unknown.
        """

        if map in map_translations:
            translated = map_translations[map]
        else:
            raise Exception(f"Unknown map: {map}")

        return translated

    def untranslateDatacenter(datacenter: str):
        """
        # Untranslate a datacenter

        This function untranslates a datacenter from common name to internal reference.

        ## Parameters

        :datacenter: The datacenter to untranslate.

        ## Example

        ```python
        LiveApex.Translator.untranslateDatacenter('')
        ```

        ## Raises

        Exception: {datacenter} | If the datacenter is unknown.
        """

        if datacenter in datacenter_untranslations:
            translated = datacenter_untranslations[datacenter]
        else:
            raise Exception(f"Unknown datacenter: {datacenter}")

        return translated

    def untranslateWeapon(weapon: str):
        """
        # Untranslate a Weapon

        This function untranslates a weapon from common name to internal reference.

        ## Parameters

        :weapon: The weapon to untranslate.

        ## Example

        ```python
        LiveApex.Translator.untranslateWeapon('Mastiff')
        ```

        ## Raises

        Exception: {weapon} | If the weapon is unknown.
        """

        if weapon in weapon_untranslations:
            translated = weapon_untranslations[weapon]
        else:
            raise Exception(f"Unknown weapon: {weapon}")

        return translated

    def untranslateMap(map: str):
        """
        # Untranslate a Map

        This function untranslates a map from common name to internal reference.

        ## Parameters

        :map: The map to untranslate.

        ## Example

        ```python
        LiveApex.Translator.untranslateMap('Kings Canyon')
        ```

        ## Raises
        Exception: {map} | If the map is unknown.
        """

        if map in map_untranslations:
            translated = map_untranslations[map]
        else:
            raise Exception(f"Unknown map: {map}")

        return translated


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: events.proto
# Protobuf Python Version: 6.30.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    1,
    '',
    'events.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x65vents.proto\x12\rrtech.liveapi\x1a\x19google/protobuf/any.proto\"*\n\x07Vector3\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"\xbe\x02\n\x06Player\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06teamId\x18\x02 \x01(\r\x12#\n\x03pos\x18\x03 \x01(\x0b\x32\x16.rtech.liveapi.Vector3\x12&\n\x06\x61ngles\x18\x04 \x01(\x0b\x32\x16.rtech.liveapi.Vector3\x12\x15\n\rcurrentHealth\x18\x05 \x01(\r\x12\x11\n\tmaxHealth\x18\x06 \x01(\r\x12\x14\n\x0cshieldHealth\x18\x07 \x01(\r\x12\x17\n\x0fshieldMaxHealth\x18\x08 \x01(\r\x12\x13\n\x0bnucleusHash\x18\t \x01(\t\x12\x14\n\x0chardwareName\x18\n \x01(\t\x12\x10\n\x08teamName\x18\x0b \x01(\t\x12\x12\n\nsquadIndex\x18\x0c \x01(\r\x12\x11\n\tcharacter\x18\r \x01(\t\x12\x0c\n\x04skin\x18\x0e \x01(\t\"b\n\x17\x43ustomMatch_LobbyPlayer\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06teamId\x18\x02 \x01(\r\x12\x13\n\x0bnucleusHash\x18\x03 \x01(\t\x12\x14\n\x0chardwareName\x18\x04 \x01(\t\"@\n\x10\x43ustomMatch_Team\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nspawnPoint\x18\x03 \x01(\x05\"D\n\x11LegendMatchStatus\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\treference\x18\x02 \x01(\t\x12\x0e\n\x06\x62\x61nned\x18\x03 \x01(\x08\"?\n\nDatacenter\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\"V\n\x07Version\x12\x11\n\tmajor_num\x18\x01 \x01(\r\x12\x11\n\tminor_num\x18\x02 \x01(\r\x12\x13\n\x0b\x62uild_stamp\x18\x03 \x01(\r\x12\x10\n\x08revision\x18\x04 \x01(\t\"B\n\rInventoryItem\x12\x10\n\x08quantity\x18\x01 \x01(\x05\x12\x0c\n\x04item\x18\x02 \x01(\t\x12\x11\n\textraData\x18\x03 \x01(\t\"v\n\x14LoadoutConfiguration\x12-\n\x07weapons\x18\x01 \x03(\x0b\x32\x1c.rtech.liveapi.InventoryItem\x12/\n\tequipment\x18\x02 \x03(\x0b\x32\x1c.rtech.liveapi.InventoryItem\"g\n\x1cPauseStateChangeNotification\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x11\n\tisPausing\x18\x02 \x01(\x08\x12\x10\n\x08timeLeft\x18\x03 \x01(\x04\x12\x0f\n\x07message\x18\x04 \x01(\t\"\x8c\x01\n\x04Init\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x13\n\x0bgameVersion\x18\x03 \x01(\t\x12*\n\napiVersion\x18\x04 \x01(\x0b\x32\x16.rtech.liveapi.Version\x12\x10\n\x08platform\x18\x05 \x01(\t\x12\x0c\n\x04name\x18\x06 \x01(\t\"\x98\x01\n\x18\x43ustomMatch_LobbyPlayers\x12\x13\n\x0bplayerToken\x18\x01 \x01(\t\x12\x37\n\x07players\x18\x02 \x03(\x0b\x32&.rtech.liveapi.CustomMatch_LobbyPlayer\x12.\n\x05teams\x18\x03 \x03(\x0b\x32\x1f.rtech.liveapi.CustomMatch_Team\"P\n\x1b\x43ustomMatch_LegendBanStatus\x12\x31\n\x07legends\x18\x03 \x03(\x0b\x32 .rtech.liveapi.LegendMatchStatus\"\xb2\x01\n\x10ObserverSwitched\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\'\n\x08observer\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06target\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12)\n\ntargetTeam\x18\x05 \x03(\x0b\x32\x15.rtech.liveapi.Player\"S\n\x12ObserverAnnotation\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x18\n\x10\x61nnotationSerial\x18\x03 \x01(\x05\"\x95\x02\n\nMatchSetup\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x0b\n\x03map\x18\x03 \x01(\t\x12\x14\n\x0cplaylistName\x18\x04 \x01(\t\x12\x14\n\x0cplaylistDesc\x18\x05 \x01(\t\x12-\n\ndatacenter\x18\x06 \x01(\x0b\x32\x19.rtech.liveapi.Datacenter\x12\x13\n\x0b\x61imAssistOn\x18\x07 \x01(\x08\x12\x15\n\ranonymousMode\x18\x08 \x01(\x08\x12\x10\n\x08serverId\x18\t \x01(\t\x12<\n\x0fstartingLoadout\x18\n \x01(\x0b\x32#.rtech.liveapi.LoadoutConfiguration\"F\n\x10GameStateChanged\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\r\n\x05state\x18\x03 \x01(\t\"_\n\x11\x43haracterSelected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\"k\n\rMatchStateEnd\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\r\n\x05state\x18\x03 \x01(\t\x12&\n\x07winners\x18\x04 \x03(\x0b\x32\x15.rtech.liveapi.Player\"\xb0\x01\n\x10RingStartClosing\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\r\n\x05stage\x18\x03 \x01(\r\x12&\n\x06\x63\x65nter\x18\x04 \x01(\x0b\x32\x16.rtech.liveapi.Vector3\x12\x15\n\rcurrentRadius\x18\x05 \x01(\x02\x12\x11\n\tendRadius\x18\x06 \x01(\x02\x12\x16\n\x0eshrinkDuration\x18\x07 \x01(\x02\"\xa0\x01\n\x13RingFinishedClosing\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\r\n\x05stage\x18\x03 \x01(\r\x12&\n\x06\x63\x65nter\x18\x04 \x01(\x0b\x32\x16.rtech.liveapi.Vector3\x12\x15\n\rcurrentRadius\x18\x05 \x01(\x02\x12\x16\n\x0eshrinkDuration\x18\x07 \x01(\x02\"]\n\x0fPlayerConnected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\"\x87\x01\n\x12PlayerDisconnected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x14\n\x0c\x63\x61nReconnect\x18\x04 \x01(\x08\x12\x0f\n\x07isAlive\x18\x05 \x01(\x08\"\x83\x01\n\x11PlayerStatChanged\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x10\n\x08statName\x18\x04 \x01(\t\x12\x10\n\x08newValue\x18\x05 \x01(\r\"u\n\x18PlayerUpgradeTierChanged\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\r\n\x05level\x18\x04 \x01(\x05\"\xad\x01\n\rPlayerDamaged\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\'\n\x08\x61ttacker\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06victim\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0e\n\x06weapon\x18\x05 \x01(\t\x12\x17\n\x0f\x64\x61mageInflicted\x18\x06 \x01(\r\"\xbd\x01\n\x0cPlayerKilled\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\'\n\x08\x61ttacker\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06victim\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12(\n\tawardedTo\x18\x05 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0e\n\x06weapon\x18\x06 \x01(\t\"\x93\x01\n\x0cPlayerDowned\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\'\n\x08\x61ttacker\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06victim\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0e\n\x06weapon\x18\x05 \x01(\t\"\x94\x01\n\x0cPlayerAssist\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12(\n\tassistant\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06victim\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0e\n\x06weapon\x18\x05 \x01(\t\"^\n\x0fSquadEliminated\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12&\n\x07players\x18\x03 \x03(\x0b\x32\x15.rtech.liveapi.Player\"y\n\x15PlayerUltimateCharged\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x14\n\x0clinkedEntity\x18\x04 \x01(\t\"\xa7\x01\n\x17GibraltarShieldAbsorbed\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\'\n\x08\x61ttacker\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06victim\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x17\n\x0f\x64\x61mageInflicted\x18\x06 \x01(\r\"\xab\x01\n\x1bRevenantForgedShadowDamaged\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\'\n\x08\x61ttacker\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12%\n\x06victim\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x17\n\x0f\x64\x61mageInflicted\x18\x06 \x01(\r\"\xa5\x01\n\x11PlayerRespawnTeam\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x11\n\trespawned\x18\x04 \x01(\t\x12\x31\n\x12respawnedTeammates\x18\x05 \x03(\x0b\x32\x15.rtech.liveapi.Player\"\x82\x01\n\x0cPlayerRevive\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12&\n\x07revived\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\"\x80\x01\n\x12\x41renasItemSelected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0c\n\x04item\x18\x04 \x01(\t\x12\x10\n\x08quantity\x18\x05 \x01(\x05\"\x82\x01\n\x14\x41renasItemDeselected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0c\n\x04item\x18\x04 \x01(\t\x12\x10\n\x08quantity\x18\x05 \x01(\x05\"}\n\x0fInventoryPickUp\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0c\n\x04item\x18\x04 \x01(\t\x12\x10\n\x08quantity\x18\x05 \x01(\x05\"\x8e\x01\n\rInventoryDrop\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0c\n\x04item\x18\x04 \x01(\t\x12\x10\n\x08quantity\x18\x05 \x01(\x05\x12\x11\n\textraData\x18\x06 \x03(\t\"z\n\x0cInventoryUse\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0c\n\x04item\x18\x04 \x01(\t\x12\x10\n\x08quantity\x18\x05 \x01(\x05\"\x87\x01\n\x0f\x42\x61nnerCollected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12(\n\tcollected\x18\x04 \x01(\x0b\x32\x15.rtech.liveapi.Player\"u\n\x11PlayerAbilityUsed\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x14\n\x0clinkedEntity\x18\x04 \x01(\t\"\x9c\x01\n\x15LegendUpgradeSelected\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x13\n\x0bupgradeName\x18\x04 \x01(\t\x12\x13\n\x0bupgradeDesc\x18\x05 \x01(\t\x12\r\n\x05level\x18\x06 \x01(\x05\"o\n\x0bZiplineUsed\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x14\n\x0clinkedEntity\x18\x04 \x01(\t\"q\n\rGrenadeThrown\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x14\n\x0clinkedEntity\x18\x04 \x01(\t\"m\n\x11\x42lackMarketAction\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x0c\n\x04item\x18\x04 \x01(\t\"Z\n\x0cWraithPortal\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\"Z\n\x0cWarpGateUsed\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\"\xa8\x01\n\x08\x41mmoUsed\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x10\n\x08\x61mmoType\x18\x04 \x01(\t\x12\x12\n\namountUsed\x18\x05 \x01(\r\x12\x14\n\x0coldAmmoCount\x18\x06 \x01(\r\x12\x14\n\x0cnewAmmoCount\x18\x07 \x01(\r\"\x82\x01\n\x0eWeaponSwitched\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12%\n\x06player\x18\x03 \x01(\x0b\x32\x15.rtech.liveapi.Player\x12\x11\n\toldWeapon\x18\x04 \x01(\t\x12\x11\n\tnewWeapon\x18\x05 \x01(\t\"o\n\x0c\x43hangeCamera\x12.\n\x03poi\x18\x01 \x01(\x0e\x32\x1f.rtech.liveapi.PlayerOfInterestH\x00\x12\x0e\n\x04name\x18\x02 \x01(\tH\x00\x12\x15\n\x0bnucleusHash\x18\x03 \x01(\tH\x00\x42\x08\n\x06target\"\x1f\n\x0bPauseToggle\x12\x10\n\x08preTimer\x18\x01 \x01(\x02\"\x19\n\x17\x43ustomMatch_CreateLobby\"*\n\x15\x43ustomMatch_JoinLobby\x12\x11\n\troleToken\x18\x01 \x01(\t\"\x18\n\x16\x43ustomMatch_LeaveLobby\"\'\n\x14\x43ustomMatch_SetReady\x12\x0f\n\x07isReady\x18\x01 \x01(\x08\"\x1d\n\x1b\x43ustomMatch_GetLobbyPlayers\"-\n\x1a\x43ustomMatch_SetMatchmaking\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\"\\\n\x13\x43ustomMatch_SetTeam\x12\x0e\n\x06teamId\x18\x01 \x01(\x05\x12\x1a\n\x12targetHardwareName\x18\x02 \x01(\t\x12\x19\n\x11targetNucleusHash\x18\x03 \x01(\t\"O\n\x16\x43ustomMatch_KickPlayer\x12\x1a\n\x12targetHardwareName\x18\x01 \x01(\t\x12\x19\n\x11targetNucleusHash\x18\x02 \x01(\t\"\x8f\x01\n\x17\x43ustomMatch_SetSettings\x12\x14\n\x0cplaylistName\x18\x01 \x01(\t\x12\x11\n\tadminChat\x18\x02 \x01(\x08\x12\x12\n\nteamRename\x18\x03 \x01(\x08\x12\x12\n\nselfAssign\x18\x04 \x01(\x08\x12\x11\n\taimAssist\x18\x05 \x01(\x08\x12\x10\n\x08\x61nonMode\x18\x06 \x01(\x08\"\x19\n\x17\x43ustomMatch_GetSettings\";\n\x17\x43ustomMatch_SetTeamName\x12\x0e\n\x06teamId\x18\x01 \x01(\x05\x12\x10\n\x08teamName\x18\x02 \x01(\t\"?\n\x19\x43ustomMatch_SetSpawnPoint\x12\x0e\n\x06teamId\x18\x01 \x01(\x05\x12\x12\n\nspawnPoint\x18\x02 \x01(\x05\"U\n\x1f\x43ustomMatch_SetEndRingExclusion\x12\x32\n\x10sectionToExclude\x18\x01 \x01(\x0e\x32\x18.rtech.liveapi.MapRegion\" \n\x1e\x43ustomMatch_GetLegendBanStatus\".\n\x18\x43ustomMatch_SetLegendBan\x12\x12\n\nlegendRefs\x18\x01 \x03(\t\"$\n\x14\x43ustomMatch_SendChat\x12\x0c\n\x04text\x18\x01 \x01(\t\"\xe6\n\n\x07Request\x12\x0f\n\x07withAck\x18\x01 \x01(\x08\x12\x14\n\x0cpreSharedKey\x18\x02 \x01(\t\x12\x30\n\tchangeCam\x18\x04 \x01(\x0b\x32\x1b.rtech.liveapi.ChangeCameraH\x00\x12\x31\n\x0bpauseToggle\x18\x05 \x01(\x0b\x32\x1a.rtech.liveapi.PauseToggleH\x00\x12I\n\x17\x63ustomMatch_CreateLobby\x18\n \x01(\x0b\x32&.rtech.liveapi.CustomMatch_CreateLobbyH\x00\x12\x45\n\x15\x63ustomMatch_JoinLobby\x18\x0b \x01(\x0b\x32$.rtech.liveapi.CustomMatch_JoinLobbyH\x00\x12G\n\x16\x63ustomMatch_LeaveLobby\x18\x0c \x01(\x0b\x32%.rtech.liveapi.CustomMatch_LeaveLobbyH\x00\x12\x43\n\x14\x63ustomMatch_SetReady\x18\r \x01(\x0b\x32#.rtech.liveapi.CustomMatch_SetReadyH\x00\x12O\n\x1a\x63ustomMatch_SetMatchmaking\x18\x0e \x01(\x0b\x32).rtech.liveapi.CustomMatch_SetMatchmakingH\x00\x12\x41\n\x13\x63ustomMatch_SetTeam\x18\x0f \x01(\x0b\x32\".rtech.liveapi.CustomMatch_SetTeamH\x00\x12G\n\x16\x63ustomMatch_KickPlayer\x18\x10 \x01(\x0b\x32%.rtech.liveapi.CustomMatch_KickPlayerH\x00\x12I\n\x17\x63ustomMatch_SetSettings\x18\x11 \x01(\x0b\x32&.rtech.liveapi.CustomMatch_SetSettingsH\x00\x12\x43\n\x14\x63ustomMatch_SendChat\x18\x12 \x01(\x0b\x32#.rtech.liveapi.CustomMatch_SendChatH\x00\x12Q\n\x1b\x63ustomMatch_GetLobbyPlayers\x18\x13 \x01(\x0b\x32*.rtech.liveapi.CustomMatch_GetLobbyPlayersH\x00\x12I\n\x17\x63ustomMatch_SetTeamName\x18\x14 \x01(\x0b\x32&.rtech.liveapi.CustomMatch_SetTeamNameH\x00\x12I\n\x17\x63ustomMatch_GetSettings\x18\x15 \x01(\x0b\x32&.rtech.liveapi.CustomMatch_GetSettingsH\x00\x12M\n\x19\x63ustomMatch_SetSpawnPoint\x18\x16 \x01(\x0b\x32(.rtech.liveapi.CustomMatch_SetSpawnPointH\x00\x12Y\n\x1f\x63ustomMatch_SetEndRingExclusion\x18\x17 \x01(\x0b\x32..rtech.liveapi.CustomMatch_SetEndRingExclusionH\x00\x12W\n\x1e\x63ustomMatch_GetLegendBanStatus\x18\x18 \x01(\x0b\x32-.rtech.liveapi.CustomMatch_GetLegendBanStatusH\x00\x12K\n\x18\x63ustomMatch_SetLegendBan\x18\x19 \x01(\x0b\x32\'.rtech.liveapi.CustomMatch_SetLegendBanH\x00\x42\t\n\x07\x61\x63tions\"\x1f\n\rRequestStatus\x12\x0e\n\x06status\x18\x01 \x01(\t\"A\n\x08Response\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12$\n\x06result\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\"M\n\x0cLiveAPIEvent\x12\x12\n\nevent_size\x18\x01 \x01(\x07\x12)\n\x0bgameMessage\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any*\x88\x01\n\x10PlayerOfInterest\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04NEXT\x10\x01\x12\x0c\n\x08PREVIOUS\x10\x02\x12\x0f\n\x0bKILL_LEADER\x10\x03\x12\x11\n\rCLOSEST_ENEMY\x10\x04\x12\x12\n\x0e\x43LOSEST_PLAYER\x10\x05\x12\x13\n\x0fLATEST_ATTACKER\x10\x06*j\n\tMapRegion\x12\x0c\n\x08TOP_LEFT\x10\x00\x12\r\n\tTOP_RIGHT\x10\x01\x12\x0f\n\x0b\x42OTTOM_LEFT\x10\x02\x12\x10\n\x0c\x42OTTOM_RIGHT\x10\x03\x12\n\n\x06\x43\x45NTER\x10\x04\x12\x11\n\rREGIONS_COUNT\x10\x05\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'events_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_PLAYEROFINTEREST']._serialized_start=9181
  _globals['_PLAYEROFINTEREST']._serialized_end=9317
  _globals['_MAPREGION']._serialized_start=9319
  _globals['_MAPREGION']._serialized_end=9425
  _globals['_VECTOR3']._serialized_start=58
  _globals['_VECTOR3']._serialized_end=100
  _globals['_PLAYER']._serialized_start=103
  _globals['_PLAYER']._serialized_end=421
  _globals['_CUSTOMMATCH_LOBBYPLAYER']._serialized_start=423
  _globals['_CUSTOMMATCH_LOBBYPLAYER']._serialized_end=521
  _globals['_CUSTOMMATCH_TEAM']._serialized_start=523
  _globals['_CUSTOMMATCH_TEAM']._serialized_end=587
  _globals['_LEGENDMATCHSTATUS']._serialized_start=589
  _globals['_LEGENDMATCHSTATUS']._serialized_end=657
  _globals['_DATACENTER']._serialized_start=659
  _globals['_DATACENTER']._serialized_end=722
  _globals['_VERSION']._serialized_start=724
  _globals['_VERSION']._serialized_end=810
  _globals['_INVENTORYITEM']._serialized_start=812
  _globals['_INVENTORYITEM']._serialized_end=878
  _globals['_LOADOUTCONFIGURATION']._serialized_start=880
  _globals['_LOADOUTCONFIGURATION']._serialized_end=998
  _globals['_PAUSESTATECHANGENOTIFICATION']._serialized_start=1000
  _globals['_PAUSESTATECHANGENOTIFICATION']._serialized_end=1103
  _globals['_INIT']._serialized_start=1106
  _globals['_INIT']._serialized_end=1246
  _globals['_CUSTOMMATCH_LOBBYPLAYERS']._serialized_start=1249
  _globals['_CUSTOMMATCH_LOBBYPLAYERS']._serialized_end=1401
  _globals['_CUSTOMMATCH_LEGENDBANSTATUS']._serialized_start=1403
  _globals['_CUSTOMMATCH_LEGENDBANSTATUS']._serialized_end=1483
  _globals['_OBSERVERSWITCHED']._serialized_start=1486
  _globals['_OBSERVERSWITCHED']._serialized_end=1664
  _globals['_OBSERVERANNOTATION']._serialized_start=1666
  _globals['_OBSERVERANNOTATION']._serialized_end=1749
  _globals['_MATCHSETUP']._serialized_start=1752
  _globals['_MATCHSETUP']._serialized_end=2029
  _globals['_GAMESTATECHANGED']._serialized_start=2031
  _globals['_GAMESTATECHANGED']._serialized_end=2101
  _globals['_CHARACTERSELECTED']._serialized_start=2103
  _globals['_CHARACTERSELECTED']._serialized_end=2198
  _globals['_MATCHSTATEEND']._serialized_start=2200
  _globals['_MATCHSTATEEND']._serialized_end=2307
  _globals['_RINGSTARTCLOSING']._serialized_start=2310
  _globals['_RINGSTARTCLOSING']._serialized_end=2486
  _globals['_RINGFINISHEDCLOSING']._serialized_start=2489
  _globals['_RINGFINISHEDCLOSING']._serialized_end=2649
  _globals['_PLAYERCONNECTED']._serialized_start=2651
  _globals['_PLAYERCONNECTED']._serialized_end=2744
  _globals['_PLAYERDISCONNECTED']._serialized_start=2747
  _globals['_PLAYERDISCONNECTED']._serialized_end=2882
  _globals['_PLAYERSTATCHANGED']._serialized_start=2885
  _globals['_PLAYERSTATCHANGED']._serialized_end=3016
  _globals['_PLAYERUPGRADETIERCHANGED']._serialized_start=3018
  _globals['_PLAYERUPGRADETIERCHANGED']._serialized_end=3135
  _globals['_PLAYERDAMAGED']._serialized_start=3138
  _globals['_PLAYERDAMAGED']._serialized_end=3311
  _globals['_PLAYERKILLED']._serialized_start=3314
  _globals['_PLAYERKILLED']._serialized_end=3503
  _globals['_PLAYERDOWNED']._serialized_start=3506
  _globals['_PLAYERDOWNED']._serialized_end=3653
  _globals['_PLAYERASSIST']._serialized_start=3656
  _globals['_PLAYERASSIST']._serialized_end=3804
  _globals['_SQUADELIMINATED']._serialized_start=3806
  _globals['_SQUADELIMINATED']._serialized_end=3900
  _globals['_PLAYERULTIMATECHARGED']._serialized_start=3902
  _globals['_PLAYERULTIMATECHARGED']._serialized_end=4023
  _globals['_GIBRALTARSHIELDABSORBED']._serialized_start=4026
  _globals['_GIBRALTARSHIELDABSORBED']._serialized_end=4193
  _globals['_REVENANTFORGEDSHADOWDAMAGED']._serialized_start=4196
  _globals['_REVENANTFORGEDSHADOWDAMAGED']._serialized_end=4367
  _globals['_PLAYERRESPAWNTEAM']._serialized_start=4370
  _globals['_PLAYERRESPAWNTEAM']._serialized_end=4535
  _globals['_PLAYERREVIVE']._serialized_start=4538
  _globals['_PLAYERREVIVE']._serialized_end=4668
  _globals['_ARENASITEMSELECTED']._serialized_start=4671
  _globals['_ARENASITEMSELECTED']._serialized_end=4799
  _globals['_ARENASITEMDESELECTED']._serialized_start=4802
  _globals['_ARENASITEMDESELECTED']._serialized_end=4932
  _globals['_INVENTORYPICKUP']._serialized_start=4934
  _globals['_INVENTORYPICKUP']._serialized_end=5059
  _globals['_INVENTORYDROP']._serialized_start=5062
  _globals['_INVENTORYDROP']._serialized_end=5204
  _globals['_INVENTORYUSE']._serialized_start=5206
  _globals['_INVENTORYUSE']._serialized_end=5328
  _globals['_BANNERCOLLECTED']._serialized_start=5331
  _globals['_BANNERCOLLECTED']._serialized_end=5466
  _globals['_PLAYERABILITYUSED']._serialized_start=5468
  _globals['_PLAYERABILITYUSED']._serialized_end=5585
  _globals['_LEGENDUPGRADESELECTED']._serialized_start=5588
  _globals['_LEGENDUPGRADESELECTED']._serialized_end=5744
  _globals['_ZIPLINEUSED']._serialized_start=5746
  _globals['_ZIPLINEUSED']._serialized_end=5857
  _globals['_GRENADETHROWN']._serialized_start=5859
  _globals['_GRENADETHROWN']._serialized_end=5972
  _globals['_BLACKMARKETACTION']._serialized_start=5974
  _globals['_BLACKMARKETACTION']._serialized_end=6083
  _globals['_WRAITHPORTAL']._serialized_start=6085
  _globals['_WRAITHPORTAL']._serialized_end=6175
  _globals['_WARPGATEUSED']._serialized_start=6177
  _globals['_WARPGATEUSED']._serialized_end=6267
  _globals['_AMMOUSED']._serialized_start=6270
  _globals['_AMMOUSED']._serialized_end=6438
  _globals['_WEAPONSWITCHED']._serialized_start=6441
  _globals['_WEAPONSWITCHED']._serialized_end=6571
  _globals['_CHANGECAMERA']._serialized_start=6573
  _globals['_CHANGECAMERA']._serialized_end=6684
  _globals['_PAUSETOGGLE']._serialized_start=6686
  _globals['_PAUSETOGGLE']._serialized_end=6717
  _globals['_CUSTOMMATCH_CREATELOBBY']._serialized_start=6719
  _globals['_CUSTOMMATCH_CREATELOBBY']._serialized_end=6744
  _globals['_CUSTOMMATCH_JOINLOBBY']._serialized_start=6746
  _globals['_CUSTOMMATCH_JOINLOBBY']._serialized_end=6788
  _globals['_CUSTOMMATCH_LEAVELOBBY']._serialized_start=6790
  _globals['_CUSTOMMATCH_LEAVELOBBY']._serialized_end=6814
  _globals['_CUSTOMMATCH_SETREADY']._serialized_start=6816
  _globals['_CUSTOMMATCH_SETREADY']._serialized_end=6855
  _globals['_CUSTOMMATCH_GETLOBBYPLAYERS']._serialized_start=6857
  _globals['_CUSTOMMATCH_GETLOBBYPLAYERS']._serialized_end=6886
  _globals['_CUSTOMMATCH_SETMATCHMAKING']._serialized_start=6888
  _globals['_CUSTOMMATCH_SETMATCHMAKING']._serialized_end=6933
  _globals['_CUSTOMMATCH_SETTEAM']._serialized_start=6935
  _globals['_CUSTOMMATCH_SETTEAM']._serialized_end=7027
  _globals['_CUSTOMMATCH_KICKPLAYER']._serialized_start=7029
  _globals['_CUSTOMMATCH_KICKPLAYER']._serialized_end=7108
  _globals['_CUSTOMMATCH_SETSETTINGS']._serialized_start=7111
  _globals['_CUSTOMMATCH_SETSETTINGS']._serialized_end=7254
  _globals['_CUSTOMMATCH_GETSETTINGS']._serialized_start=7256
  _globals['_CUSTOMMATCH_GETSETTINGS']._serialized_end=7281
  _globals['_CUSTOMMATCH_SETTEAMNAME']._serialized_start=7283
  _globals['_CUSTOMMATCH_SETTEAMNAME']._serialized_end=7342
  _globals['_CUSTOMMATCH_SETSPAWNPOINT']._serialized_start=7344
  _globals['_CUSTOMMATCH_SETSPAWNPOINT']._serialized_end=7407
  _globals['_CUSTOMMATCH_SETENDRINGEXCLUSION']._serialized_start=7409
  _globals['_CUSTOMMATCH_SETENDRINGEXCLUSION']._serialized_end=7494
  _globals['_CUSTOMMATCH_GETLEGENDBANSTATUS']._serialized_start=7496
  _globals['_CUSTOMMATCH_GETLEGENDBANSTATUS']._serialized_end=7528
  _globals['_CUSTOMMATCH_SETLEGENDBAN']._serialized_start=7530
  _globals['_CUSTOMMATCH_SETLEGENDBAN']._serialized_end=7576
  _globals['_CUSTOMMATCH_SENDCHAT']._serialized_start=7578
  _globals['_CUSTOMMATCH_SENDCHAT']._serialized_end=7614
  _globals['_REQUEST']._serialized_start=7617
  _globals['_REQUEST']._serialized_end=8999
  _globals['_REQUESTSTATUS']._serialized_start=9001
  _globals['_REQUESTSTATUS']._serialized_end=9032
  _globals['_RESPONSE']._serialized_start=9034
  _globals['_RESPONSE']._serialized_end=9099
  _globals['_LIVEAPIEVENT']._serialized_start=9101
  _globals['_LIVEAPIEVENT']._serialized_end=9178
# @@protoc_insertion_point(module_scope)



import asyncio
import websockets

## LiveApex WebSocket Server ##
# This starts the WebSocket server for the LiveApex library #

connected_clients = set()

async def echo(websocket, path):
    connected_clients.add(websocket)
    try:
        async for message in websocket:
            # Broadcast the message to all connected clients
            tasks = [asyncio.create_task(client.send(message)) for client in connected_clients]
            await asyncio.gather(*tasks)

    except websockets.exceptions.ConnectionClosedOK:
        pass

    finally:
        connected_clients.remove(websocket)

async def main():
    try:
        async with websockets.serve(echo, '127.0.0.1', '7777', open_timeout=None, ping_timeout=None):
            print(f"WebSocket server started on ws://127.0.0.1:7777")
            await asyncio.Future() # Run forever
    except OSError as e: # Another websocket instance is already running
        if '10048' in str(e):
            raise Exception("[LiveApexSocket] existingInstance: Another websocket instance is already running")

        else:
            raise e

if __name__ == "__main__":
    asyncio.run(main())


from setuptools import setup, find_packages

version = '1.1.2'
description = 'A Python library for interacting with the Apex Legends LiveAPI.'
long_description = 'A Python library for interacting with the Apex Legends LiveAPI. Includes recieving game events and sending events to interact with the game.'

setup(
    name='LiveApex',
    version=version,
    author='Catotron',
    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[
        'protobuf==6.30.1',
        'websockets==12.0'
    ],

    keywords='apex legends, live api, python, library',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3',
        'Operating System :: Microsoft :: Windows',
    ],
    python_requires='>=3.12'
)


import asyncio
import websockets
import os
import site

from google.protobuf.any_pb2 import Any
from google.protobuf import symbol_database
from google.protobuf.json_format import MessageToDict
from . import events_pb2

### LiveApex Core Functions ###
# These functions are essential for the LiveApex library to work #

class Core:
    """
    # Core

    This class contains functions to start the WebSocket server and listener.
    """

    async def startLiveAPI():
        """
        # Start the LiveAPI WebSocket server

        This function starts the LiveAPI WebSocket server. It is used to connect to the game to send/receive events.

        ## Example

        ```python
        LiveApex.Core.startLiveAPI()
        ```
        """

        # Get server.py path
        server_path = os.path.join(site.getsitepackages()[0], "Lib", "site-packages", "LiveApex", "server.py")

        # start server.py subprocess
        process = await asyncio.create_subprocess_exec(
            "python", server_path,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE
        )

        print("[LiveApexCore] Starting WebSocket Server")

        # Read the output and error streams
        async def read_stream(stream, callback):
            while True:
                line = await stream.readline()
                if not line:
                    break
                callback(line.decode().strip())

        # Define output and error streams
        stdout_task = asyncio.create_task(read_stream(process.stdout, lambda x: print(f"[LiveApexSocket] {x}")))
        stderr_task = asyncio.create_task(read_stream(process.stderr, lambda x: print(f"[LiveApexSocket] [ERROR] {x}")))

        # Keep socket process running
        await process.wait()

        # Close streams after socket ends
        stdout_task.cancel()
        stderr_task.cancel()

        # Catch any exceptions that happen when tasks end
        try:
            try:
                await stdout_task
                await stderr_task
            except asyncio.CancelledError as e:
                print(f"[LiveApexCore] Error: {e}")
                pass
        except Exception as e:
            print(f"[LiveApexCore] Error: {e}")
            pass

        print("[LiveApexCore] WebSocket Server Process Ended")

    # Define how websocket events are handled
    async def startListener(callback):
        async with websockets.connect(f"ws://127.0.0.1:7777") as websocket:
            print("[LiveApexCore] Started WebSocket Listener")
            previous_message = None
            async for message in websocket:
                if message == previous_message: # TEMP PATCH: Ignore duplicate messages (Recent patch seems to have broken the LiveAPI, so each event is sent twice)
                    continue
                previous_message = message
                decoded = Core.decodeSocketEvent(message)

                await callback(decoded)

    def decodeSocketEvent(event: Any):
        """
        # Decode a Socket Event

        This function decodes a socket event. Used to convert socket events to a `dict`.

        ## Parameters

        :event: The event to decode.

        ## Returns

        The decoded event as `dict`.

        ## Example

        ```python
        decodeSocketEvent(event)
        ```
        """

        try:
            # Parse event
            live_api_event = events_pb2.LiveAPIEvent()
            live_api_event.ParseFromString(event)

            try:
                result_type = live_api_event.gameMessage.TypeName()

                # Filters
                if result_type != "":
                    if result_type == "rtech.liveapi.Response": # Response messages
                        msg_result = symbol_database.Default().GetSymbol(result_type)()
                        live_api_event.gameMessage.Unpack(msg_result)
                        result = MessageToDict(msg_result)

                        if result['success'] == True:
                            if "CustomMatch_SetSettings" in result['result']['@type']: # if setting is False that setting is not sent
                                playlistName = result['result']['playlistName']
                                adminChat = result['result'].get('adminChat', False)
                                teamRename = result['result'].get('teamRename', False)
                                selfAssign = result['result'].get('selfAssign', False)
                                aimAssist = result['result'].get('aimAssist', False)
                                anonMode = result['result'].get('anonMode', False)
                                result = {
                                    "playListName": playlistName,
                                    "adminChat": adminChat,
                                    "teamRename": teamRename,
                                    "selfAssign": selfAssign,
                                    "aimAssist": aimAssist,
                                    "anonMode": anonMode,
                                }
                                return result

                    else: # LiveAPIEvents
                        msg_result = symbol_database.Default().GetSymbol(result_type)()
                        live_api_event.gameMessage.Unpack(msg_result)
                        result = MessageToDict(msg_result)
                        return result

                else: # Assume sending to websocket
                    return None

            except Exception as e:
                print(f"[LiveApexCore] Error decoding socket event: {e}")
                return None

        except Exception as e:
            print(f"[LiveApexCore] Error parsing event: {e}")
            return None